/***************************************************************************
                          calltreedlg.cpp
                             -------------------
    begin                : Fri Jul 18 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qclipboard.h>
#include <qfile.h>
#include <qtoolbutton.h>
#include <qspinbox.h>
#include <klocale.h>
#include <kfiledialog.h>
#include "calltreedlg.h"
#include "graphwidget.h"
#include "kscopepixmaps.h"
#include "kscopeconfig.h"
#include "graphprefdlg.h"

/** File Name index for the file name generation */
int CallTreeDlg::s_nFileNameIndex = 0;

/**
 * Class constructor.
 * @param	pParent	The parent widget
 * @param	szName	The widget's name
 */
CallTreeDlg::CallTreeDlg(QWidget* pParent, const char* szName) :
	CallTreeLayout(pParent, szName)
{
	// Make sure the dialog object is deleted when the dialog is closed
	setWFlags(getWFlags() | WDestructiveClose);

	// Set button pixmaps
	m_pSaveButton->setPixmap(GET_PIXMAP(ButtonSaveAs));
	m_pZoomInButton->setPixmap(GET_PIXMAP(ButtonZoomIn));
	m_pZoomOutButton->setPixmap(GET_PIXMAP(ButtonZoomOut));
	m_pRotateButton->setPixmap(GET_PIXMAP(ButtonRotate));
	m_pPrefButton->setPixmap(GET_PIXMAP(ButtonPref));
	
	// Open the location of a call
	connect(m_pGraphWidget, SIGNAL(lineRequested(const QString&, uint)),
		this, SIGNAL(lineRequested(const QString&, uint)));
}

/**
 * Class destructor.
 */
CallTreeDlg::~CallTreeDlg()
{
}

/**
 * @param	sFunc	The function to use as the root of the call tree
 */
void CallTreeDlg::setRoot(const QString& sFunc)
{
	// Generate unique file name to save call tree later
	m_sFileName = sFunc;
	m_sFileName.replace(' ', '_');
	m_sFileName += QString::number(++s_nFileNameIndex);

	// Create an initial node
	m_pGraphWidget->setRoot(sFunc);
	m_pGraphWidget->draw();
}

/**
 * Informs the call tree manager that this object should be removed from the
 * list of call tree dialogues.
 * The close event is received when the dialogue is explicitly closed by the
 * user. This dialogue will not appear when the project is reopened, and it
 * is therefore safe to delete the graph file at this point.
 * @param	pEvent	Information on the closing event 
 */
void CallTreeDlg::closeEvent(QCloseEvent* pEvent)
{
	if (!m_sFilePath.isEmpty())
		QFile::remove(m_sFilePath);
		
	emit closed(this);
	QDialog::closeEvent(pEvent);
}

/**
 * Restores a call tree from the given call tree file.
 * NOTE: The call tree file is deleted when loading is complete.
 * @param	sProjPath	The full path of the project directory
 * @param	sFileName	The name of the call tree file to load
 * @return	true if successful, false otherwise
 */
bool CallTreeDlg::load(const QString& sProjPath, const QString& sFileName)
{
	QString sPath;
	
	// Load the graph from the given file
	sPath = sProjPath + "/" + sFileName;
	if (!m_pGraphWidget->load(sPath))
		return false;
	
	// Store the file name
	m_sFileName = sFileName;
	m_sFilePath = sPath;
	
	// Draw the graph
	m_pGraphWidget->draw();
	return true;
}

/**
 * Writes the contents of the call tree dialog to a call tree file.
 * This method is called for call trees before the owner project is
 * closed.
 * @param	sProjPath	The full path of the project directory
 */
void CallTreeDlg::store(const QString& sProjPath)
{
	QString sPath;
	
	sPath = sProjPath + "/" + m_sFileName;
	m_sFilePath = sPath;
	m_pGraphWidget->save(sPath, true);
}

/**
 * Saves the graph to a dot file.
 * The user is prompted for a name to use for the file, and then graph
 * widget writes its information to this file (using the dot language).
 * This slot is connected to the clicked() signal of the "Save As..." button.
 */
void CallTreeDlg::slotSaveClicked()
{
	QString sFile;
	
	// Prompt the user for a file name
	sFile = KFileDialog::getSaveFileName(":kscope");
	
	// Save the graph to a file (unless the user did not give a file name)
	if (!sFile.isEmpty())
		m_pGraphWidget->save(sFile, false);
}

/**
 * Increases the zoom factor of the graph.
 * This slot is connected to the clicked() signal of the "Zoom In" button.
 */
void CallTreeDlg::slotZoomInClicked()
{
	m_pGraphWidget->zoom(true);
	m_pGraphWidget->draw();
}

/**
 * Decreases the zoom factor of the graph.
 * This slot is connected to the clicked() signal of the "Zoom Out" button.
 */
void CallTreeDlg::slotZoomOutClicked()
{
	m_pGraphWidget->zoom(false);
	m_pGraphWidget->draw();
}

/**
 * Changes the graph's layout direction.
 * This slot is connected to the clicked() signal of the "Rotate" button.
 */
void CallTreeDlg::slotRotateClicked()
{
	m_pGraphWidget->rotate();
	m_pGraphWidget->draw();
}

/**
 * Opens the call graph preferences dialogue.
 * This slot is connected to the clicked() signal of the "Preferences" button.
 */
void CallTreeDlg::slotPrefClicked()
{
	GraphPrefDlg dlg;
	int nMaxNodeDegree;
	
	if (dlg.exec() == QDialog::Accepted) {
		nMaxNodeDegree = dlg.getMaxNodeDegree();
		Config().setGraphMaxNodeDegree(nMaxNodeDegree);
		m_pGraphWidget->setMaxNodeDegree(nMaxNodeDegree);
	}
}

#include "calltreedlg.moc"
