/***************************************************************************
                             graphwidget.h
                             -------------------
    begin                : Fri Jun 10 2005
    copyright            : (C) 2005 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GRAPHWIDGET_H
#define GRAPHWIDGET_H

#include <qcanvas.h>
#include <qpopupmenu.h>
#include <qdict.h>
#include "cscopefrontend.h"
#include "graphviz.h"

class GraphNode;
class GraphEdge;

/**
 * A widget that displays call tree graphs generated by graphviz.
 * This class is based on a QCanvasView widget, and displays two types of
 * canvas items: GraphNode, which draws the name of a function inside a
 * polygon, and ArrowEdge, which is a directed graph edge shaped like an
 * arrow.
 * The call tree graph is populated using the addNode() method. The first
 * call creates a root node. Subsequent calls add nodes which represent either
 * functions called by a previously inserted node, or that are calling such
 * a node. A directed edge is created to depict the relationship between a
 * node and its parent.
 * Drawing is done through the embedded Agraph_t object (a graph, as defined
 * by the graphviz libraray). When the draw() method is called, the graph is
 * layed out in memory. The class then uses the CodeGenerator class to
 * obtain a set of instructions on how to actually draw the graph. These
 * instructions are used to create the appropriate canvas items.
 * An application _must_ call GraphWidget::init() before attempting to use
 * this class. It should also call GraphWidget::fini() when it no longer needs
 * any of these widgets.
 * @author Elad Lahav
 */
class GraphWidget : public QCanvasView
{
	Q_OBJECT
	
public:
	GraphWidget(QWidget* pParent = 0, const char* szName = 0);
	~GraphWidget();
	
	/**
	 * Information on a function call, as produced by a Cscope query.
	 * This structure is used for adding calls to the graph.
	 * @see addCall()
	 */
	struct CallData
	{
		/** The name of the calling function. */
		QString m_sCaller;
		
		/** The name of the called function. */
		QString m_sCallee;
		
		/** Path of the file in which the call appears. */
		QString m_sFile;
		
		/** The line number of the call. */
		QString m_sLine;
		
		/** The call's text. */
		QString m_sText;
	};

	/** Graph orientation values. */
	enum Orientation { Portrait, Landscape };

	void reset();
	void setRoot(const QString&);
	void addCall(const CallData&);
	void addMultiCall(const QString&, bool);
	void draw();
	bool load(const QString&);
	void save(const QString&, bool);
	void zoom(bool);
	void rotate();
	QString getTip(const QPoint&, QRect&);
	
	void resize(int, int);
	void startNode(Agnode_t*);
	void endNode();
	void startEdge(Agedge_t*);
	void endEdge();
	void polygon(const QPointArray&);
	void curve(const QPointArray&);
	void text(const QString&, const QPoint&);
	
	/**
	 * @return	The internal graph object
	 */
	Agraph_t* getGraph() { return m_pGraph; }
	
	/**
	 * Adjusts the maximal number of calling/called functions shown for
	 * every node (@see m_nMaxNodeDegree).
	 * @param	nMaxNodeDegree	The new value to set
	 */
	void setMaxNodeDegree(int nMaxNodeDegree) { m_nMaxNodeDegree =
		nMaxNodeDegree; }
	
signals:
	/**
	 * Emitted when the user makes a request to view the contents of a
	 * location in the source code.
	 * This can be the location of a call, the definition of a function,
	 * etc.
	 * @param	sPath	The full path of the file to show
	 * @param	nLine	The line number in this file
	 */
	void lineRequested(const QString& sPath, uint nLine);
	
protected:
	
	virtual void drawContents(QPainter*, int, int, int, int);
	virtual void contentsMousePressEvent(QMouseEvent*);
	
private:
	/** The canvas object that controls this view. All canvas items are
		created as children of this object.	*/
	QCanvas* m_pCanvas;
	
	/** A graphviz graph object, used to lay out and draw the call tree
		graph. */
	Agraph_t* m_pGraph;
	
	/** A Cscope process to use for running queries. */
	CscopeFrontend* m_pCscope;
	
	/** Displays query progress information. */
	CscopeProgress m_progress;
	
	/** Remembers the function the was last queried for calling/called
		functions. */
	QString m_sQueriedFunc;
	
	/** Remembers whether the last query was for calling or called
		functions. */
	bool m_bCalled;
	
	/** The item currently being drawn. This item is set by the startNode()
		and startEdge() methods. */
	QCanvasPolygonalItem* m_pCurItem;
	
	/** The node over which the popup menu has been invoked. */
	QCanvasPolygonalItem* m_pMenuItem;
	
	/** A popup menu that appears when a node is right-clicked. */
	QPopupMenu* m_pNodePopup;
	
	/** A popup menu that appears when a node is right-clicked. */
	QPopupMenu* m_pMultiCallPopup;
	
	/** A popup menu that appears when an edge is right-clicked. */
	QPopupMenu* m_pEdgePopup;

	/** The zoom factor for the graph. */
	double m_dZoom;
	
	/** Maximal number of in/out edges per node. If this number is exceeded,
		the graph shows a single "mulyi-call" node. */
	int m_nMaxNodeDegree;
	
	void setNodeAttrib();
	void removeEdges(GraphNode*, bool);
	void showNodeMenu(GraphNode*, const QPoint&);
	void showEdgeMenu(GraphEdge*, const QPoint&);
	
private slots:
	void slotDataReady(FrontendToken*);
	void slotProgress(int, int);
	void slotFinished(uint);
	void slotAborted();
	void slotShowCalled();
	void slotListCalled();
	void slotHideCalled();
	void slotShowCalling();
	void slotListCalling();
	void slotHideCalling();
	void slotFindDef();
	void slotRemoveNode();
	void slotMultiCallDetails();
	void slotOpenCall();
};

#endif
